+++
date = "2019-10-24"
weight = 100

title = "Guidelines Gitlab-based packaging workflow"

aliases = [
    "/old-wiki/Guidelines/Gitlab-based_packaging_workflow"
]
+++

# GitLab-based packaging workflow

Packaging in Apertis is handled through a set of git repositories hosted
on GitLab containing the sources for every packages. GitLab CI pipelines
take care of propagating updates to OBS to build and publish the
resulting binaries.

## Repository contents

The packaging git repositories follow the
[DEP-14](http://dep.debian.net/deps/dep14/) Git layout specification
with the following conventions:

  - `upstream/latest` branches with the unpacked upstream project code
    from the debian package)
      - FIXME: change to: `upstream/${UPSTREAM_DISTRIBUTION}` branches
        with the unpacked upstream project code from the debian package
        (e.g. `upstream/buster`)
  - `debian/${UPSTREAM_DISTRIBUTION}` branches with the Debian changes
    on top of the upstream project code (e.g. `debian/buster`)
  - `debian/${UPSTREAM_VERSION}` tags (e.g. `apertis/2.48.0-1`)
  - `apertis/${APERTIS_RELEASE}` branches where the debian branches
    above get merged with the Apertis packaging (e.g.
    `apertis/v2019pre`)
  - `apertis/${APERTIS_VERSION}` tags (e.g. `apertis/2.48.0-1co1`)
  - `pristine-lfs` branch containing the information to fetch the
    original tarballs stored on Git-LFS via
    [pristine-lfs](https://gitlab.apertis.org/infrastructure/pristine-lfs)

In addition, all the packaging repositories contains a
`debian/apertis/gitlab-ci.yml` file to enable the CI pipeline which
generates source packages and uploads them to OBS.

## Development environment

All the instructions below assume an Apertis development enviroment:
either boot the Apertis [SDK]( {{< ref "/sdk.md" >}} ) or enter the
`apertis-*-package-source-builder` Docker container:

    APERTIS_RELEASE=v2019pre
    docker pull docker-registry.apertis.org/apertis/apertis-${APERTIS_RELEASE}-package-source-builder
    docker run -it --rm --env HOME -w "$(pwd)" -v "$HOME:$HOME" -v "$HOME:/root" --security-opt label=disable apertis-${APERTIS_RELEASE}-package-source-builder bash

## How to manually sync an Apertis package with a new version

Upstream updates are usually handled automatically by the
[`ci-update-from-upstream.yml`](https://gitlab.apertis.org/infrastructure/ci-package-builder/)
Continous Integration pipeline, which fetches upstream packages, merges them
with the Apertis contents and directly creates Merge Requests to be reviewed
by maintainers.

However, in some cases it is necessary to manuall pull upstream contents:
1. the CI failed to merge the upstream update with the downstream changes, so a
   developer must reproduce the merge and fix the conflicts
1. for some reason it in necessary to manually pull updates from a new upstream
   distribution/suite

The steps below can guide you on manually pulling upstream updates:

1.  On GitLab, [fork the
    project](https://docs.gitlab.com/ce/user/project/repository/forking_workflow.html)
    to your personal namespace.

1.  Check out the source repository

        GITLAB_USER=${USER}
        UPSTREAM_DISTRIBUTION=buster
        MIRROR=https://deb.debian.org/debian
        PACKAGE=glib2.0

        git clone git@gitlab.apertis.org:${GITLAB_USER}/${PACKAGE}
        cd ${PACKAGE}

1.  Only if moving to a new upstream distribution, prepare the upstream base
    branches using the current upstream branches as a base:

        UPSTREAM_DISTRIBUTION=bullseye
        UPSTREAM_BASE=buster
        git push origin origin/upstream/${UPSTREAM_BASE}:upstream/${UPSTREAM_DISTRIBUTION}
        git push origin origin/debian/${UPSTREAM_BASE}:debian/${UPSTREAM_DISTRIBUTION}

1.  Sync the upstream contents

        apertis-pkg-pull-updates --package=$PACKAGE --upstream=$UPSTREAM_DISTRIBUTION --mirror=$MIRROR
        git push origin --follow-tags pristine-lfs upstream/${UPSTREAM_DISTRIBUTION} debian/${UPSTREAM_DISTRIBUTION}

1.  Merge the upstream changes

        PROPOSED_BRANCH=wip/$GITLAB_USER/update-from-${UPSTREAM_DISTRIBUTION}
        TARGET_BRANCH=apertis/v2021dev1
        git checkout -b ${PROPOSED_BRANCH} ${TARGET_BRANCH}
        apertis-pkg-merge-updates --upstream=debian/${UPSTREAM_DISTRIBUTION} --downstream=${PROPOSED_BRANCH}

1.  Only if the merge fails, fix the conflicts and continue

        git status
        # fix conflicts
        git merge --continue
        dch --local co --no-auto-nmu

1.  Check `debian/changelog` and update it if needed by listing all the remaining Apertis changes

        $EDITOR debian/changelog
        git commit --signoff --amend debian/changelog

1.  Push your changes for review

        git push origin --follow-tags pristine-lfs upstream/${UPSTREAM_DISTRIBUTION} debian/${UPSTREAM_DISTRIBUTION}
        git push -o merge_request.create

## How to work with downstream packaging changes

The packaging repositories follow the [Apertis contribution
process](https://designs.apertis.org/latest/contribution-process.html).

However, the deb packaging format does not allow changes outside of the
`debian/` folder: changes to the upstream files should be carried as
patches, shipped in `debian/patches` folder.

To make working with those patches a bit easier for those used to `git
rebase`, [the `gbp pq` tool provides a way to convert patches to git
branches and
back](https://honk.sigxcpu.org/projects/git-buildpackage/manual-html/gbp.patches.html).

## How to issue a release

Once all the other changes have been reviewed and landed following the
[Apertis contribution
process](https://designs.apertis.org/latest/contribution-process.html) a
package release need to be issues to land the update to the package pool
in OBS.

The only things needed to do so are updating the `debian/changelog` file
and pushing the tagged commit to the Apertis release branch:

1.  Finalize the changelog, commit and tag

        GBP_CONF_FILES=/etc/git-buildpackage/gbp.conf gbp dch -R -D apertis --debian-branch=apertis/${APERTIS_RELEASE}

        git add ./debian/changelog

        git commit --signoff -m "Release $(dpkg-parsechangelog -S source) $(dpkg-parsechangelog -S version)"

        GBP_CONF_FILES=/etc/git-buildpackage/gbp.conf gbp buildpackage --git-tag --git-tag-only --git-debian-branch=apertis/${APERTIS_RELEASE} --git-debian-tag='apertis/%(version)s'

2.  Push the release commit and tag

        git push origin HEAD apertis/$(dpkg-parsechangelog -S version)

## How to add a new packaging repository

1.  Initialize a git source repository and import the upstream tarball

        UPSTREAM_DISTRIBUTION=buster
        UPSTREAM_TARBALL=https://github.com/xfce-mirror/thunarx-python/archive/thunarx-python-0.5.1.tar.gz
        PACKAGE=thunarx-python
        COMPONENT=target

        git init ${PACKAGE}
        cd ${PACKAGE}

        gbp import-orig ${UPSTREAM_TARBALL} --debian-branch=debian/${UPSTREAM_DISTRIBUTION} --upstream-branch=upstream/${UPSTREAM_DISTRIBUTION} --no-pristine-tar

2.  Create pristine-lfs branch and commit upstream tarball

        pristine-lfs commit ../thunarx-python_0.5.1.orig.tar.gz

3.  Add generic debian/ packaging to debian/buster

        cp -a ~/sample-debian-packaging-folder/ debian/
        git add debian/
        git commit -m "Add initial debian/ packaging" debian/

4.  Add Apertis specific packaging changes

        git checkout -b apertis/${APERTIS_RELEASE} debian/${UPSTREAM_DISTRIBUTION}

        # Add Apertis specific changes

        git commit -m "Apertis specific changes" debian/

5.  Prepare changelog for release

        dch -l co

        # in the editor add some contents to the created changelog entry, leaving it UNRELEASED, for instance:
        #
        # thunarx-python (0.5.1-1co1) UNRELEASED; urgency=medium
        #
        #  * New package import
        #  * Apertis specific changes:
        #    - Add patch to introduce an apertis specific package

        git add ./debian/changelog

        git commit --signoff -m "Sync with Debian ${UPSTREAM_DISTRIBUTION} ${PACKAGE} ${UPSTREAM_VERSION}"

6.  Push your changes to gitlab

        # Create a repository on the gitlab infra under proper group. Eg: git@gitlab.apertis.org:pkg/development/thunarx-python.git
        # And then add the same remote to your git repository to push your changes to the remote server.

        git remote add origin git@gitlab.apertis.org:pkg/development/thunarx-python.git
        git push origin --all
